// DumpConsumer.cpp: implementation of the CDumpConsumer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Impl.h"
#include "DumpConsumer.h"
#include "NodeFactory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDumpConsumer::CDumpConsumer()
{
	m_pNodeFactory = new CNodeFactory();
}

CDumpConsumer::~CDumpConsumer()
{
	delete m_pNodeFactory;
}

void CDumpConsumer::startModel(void)
{
	printf("Start Model\n");
}

void CDumpConsumer::endModel(void)
{
	printf("End Model\n");
}

void CDumpConsumer::addStatement(IStatement *pStatement)
{
	printf(
		"{ %s, %s, %s }\n",
		(LPCTSTR) pStatement->predicate()->getLabel(),
		(LPCTSTR) pStatement->subject()->getLabel(),
		(LPCTSTR) pStatement->object()->getLabel()
	);
}

INodeFactory *CDumpConsumer::getNodeFactory(void)
{
	return m_pNodeFactory;
}


/*
 * From ISAXErrorHandler
 */

HRESULT STDMETHODCALLTYPE CDumpConsumer::error(
            /* [in] */ ISAXLocator __RPC_FAR *pLocator,
            /* [in] */ unsigned short * pwchErrorMessage,
			/* [in] */ HRESULT errCode)
{
	wprintf(L"Error: %s", pwchErrorMessage);
	return S_OK;
}


HRESULT STDMETHODCALLTYPE CDumpConsumer::fatalError(
            /* [in] */ ISAXLocator __RPC_FAR *pLocator,
            /* [in] */ unsigned short * pwchErrorMessage,
			/* [in] */ HRESULT errCode)
{
	wprintf(L"Error: %s", pwchErrorMessage);
	return E_FAIL;
}


HRESULT STDMETHODCALLTYPE CDumpConsumer::ignorableWarning(
            /* [in] */ ISAXLocator __RPC_FAR *pLocator,
            /* [in] */ unsigned short * pwchErrorMessage,
			/* [in] */ HRESULT errCode)
{
	wprintf(L"Error: %s", pwchErrorMessage);
	return S_OK;
}


/*
 * From IUnknown
 */

long __stdcall CDumpConsumer::QueryInterface(const struct _GUID &riid,void ** ppvObject)
{
    return 0;
}


unsigned long __stdcall CDumpConsumer::AddRef()
{
    return 0;
}


unsigned long __stdcall CDumpConsumer::Release()
{
    return 0;
}
