// GenericParser.cpp: implementation of the GenericParser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Impl.h"
#include "GenericParser.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGenericParser::CGenericParser()
{
	CoInitialize(NULL);
	m_nNumInParent = 0;
	m_pCurrent = NULL;
	m_pErrorHandler = NULL;
}

CGenericParser::~CGenericParser()
{
	CoUninitialize();
	delete m_pCurrent;
}


CElement* CGenericParser::createElement(void)
{
	return new CElement();
}

HRESULT CGenericParser::createXMLReader(LPCTSTR pwchClassName, ISAXXMLReader** pXMLReader)
{
	*pXMLReader = NULL;

	HRESULT hr = CoCreateInstance(
		__uuidof(SAXXMLReader), 
		NULL, 
		CLSCTX_ALL, 
		__uuidof(ISAXXMLReader), 
		(void **)pXMLReader);
	return hr;
}

HRESULT CGenericParser::setErrorHandler(ISAXErrorHandler *pHandler)
{
	m_pErrorHandler = pHandler;
	return S_OK;
}


HRESULT STDMETHODCALLTYPE CGenericParser::_endElement(
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName)
{
	m_nNumInParent = m_pCurrent->getNumInParent() + 1;
	CElement *pTemp = m_pCurrent->getParent();
	delete m_pCurrent;
	m_pCurrent = pTemp;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CGenericParser::_startElement(
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ ISAXAttributes __RPC_FAR *pAttributes)
{
	CElement *pEl = createElement();

	/*
	 * Create an element to hold all of this element's information
	 */

	pEl->putName(new CQName(pwchNamespaceUri, cchNamespaceUri, pwchLocalName, cchLocalName));
	pEl->putNumInParent(m_nNumInParent);
	m_nNumInParent = 1;

	/*
	 * Copy the attributes
	 */

	int nNumAttributes,
		ncNamespace,
		ncLocalName,
		ncRawName,
		ncValue;

	wchar_t *sNamespace,
			*sLocalName,
			*sRawName,
			*sValue;

	pAttributes->getLength(&nNumAttributes);
	for (int i = 0; i < nNumAttributes; i++)
	{
		pAttributes->getName(i, &sNamespace, &ncNamespace, &sLocalName, &ncLocalName, &sRawName, &ncRawName);
		if (ncNamespace || ncLocalName)
		{
			pAttributes->getValue(i, &sValue, &ncValue);
			CQName qname(sNamespace, ncNamespace, sLocalName, ncLocalName);
			pEl->putAttribute(&qname, sValue, ncValue);
		}
	} // for ( each attribute on this element )

	/*
	 * Add the element as a child of the last element
	 */

	if (!m_pCurrent)
		m_pCurrent = pEl;
	else
	{
		m_pCurrent->putChild(pEl);
		pEl->putParent(m_pCurrent);
		m_pCurrent = pEl;
	}
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CGenericParser::endElement(
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ wchar_t __RPC_FAR *pwchRawName,
            /* [in] */ int cchRawName)
{
	//wprintf(L"End element: %s\n", m_pelCurrent->getName());
	_endElement(pwchNamespaceUri, cchNamespaceUri, pwchLocalName, cchLocalName);
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CGenericParser::startElement(
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ wchar_t __RPC_FAR *pwchRawName,
            /* [in] */ int cchRawName,
            /* [in] */ ISAXAttributes __RPC_FAR *pAttributes)
{
	_startElement(pwchNamespaceUri, cchNamespaceUri, pwchLocalName, cchLocalName, pAttributes);
    return S_OK;
}
