// RDFAPI.h: interface for the IRDFConsumer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RDFAPI_H__6A721C45_BAB0_4AFF_8D50_E9256F5826DD__INCLUDED_)
#define AFX_RDFAPI_H__6A721C45_BAB0_4AFF_8D50_E9256F5826DD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "QName.h"

class IRDFNode
{
public:
	virtual CString getLabel(void) = 0;
};


class ILiteral : public IRDFNode
{

};


class IResource : public IRDFNode
{
public:
	virtual CString getLocalName(void) = 0;
	virtual CString getNamespace(void) = 0;
	virtual CString getURI(void) = 0;
};


class IStatement : public IResource
{
public:
	virtual IRDFNode *object(void) = 0;
	virtual IResource *predicate(void) = 0;
	virtual IResource *subject(void) = 0;
};


class IModel : public IResource
{
public:
    virtual HRESULT add(IStatement **pStatement) = 0;
    //virtual HRESULT elements(pointer to enumeration object) = 0;
    virtual HRESULT size(int *pnSize) = 0;
};


class IRDFSerializer
{
public:
    virtual HRESULT serialize(IModel **pModel) = 0;
};


class IInputSource
{
};


class INodeFactory
{
public:
	virtual ILiteral *createLiteral(LPCTSTR lpstrStr) = 0;
	virtual IResource *createResource(LPCTSTR lpstrStr) = 0;
	virtual IResource *createResource(LPCTSTR pstrNamespace, LPCTSTR pstrLocalName) = 0;
	virtual IStatement *createStatement(IResource *pSubject, IResource *pPredicate, IRDFNode *pObject) = 0;
};


class IRDFConsumer
{
public:
    virtual void startModel(void) = 0;
    virtual void endModel(void) = 0;
    virtual void addStatement(IStatement *pStatement) = 0;
	virtual INodeFactory *getNodeFactory(void) = 0;
};


class IRDFParser
{
public:
    virtual HRESULT parse(LPCTSTR pSource, IRDFConsumer *pConsumer) = 0;
	//virtual void setErrorHandler(ISAXErrorHandler *pHandler) = 0;
};

#endif // !defined(AFX_RDFAPI_H__6A721C45_BAB0_4AFF_8D50_E9256F5826DD__INCLUDED_)
