// RDFContentHandler.cpp: implementation of the RDFContentHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RDFContentHandler.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CResourceStack::~CResourceStack()
{
	while (!this->IsEmpty())
	{
		delete this->GetHead();
		this->RemoveHead();
	}
}

CElementStack::~CElementStack()
{
	while (!this->IsEmpty())
	{
		delete this->GetHead();
		this->RemoveHead();
	}
}

CParseItemStack::~CParseItemStack()
{
	while (!this->IsEmpty())
	{
		delete this->GetHead();
		this->RemoveHead();
	}
}

CResource::CResource(const wchar_t *pwchResource)
{
	m_pResourceName = new wchar_t[wcslen(pwchResource)];
	wcscpy(m_pResourceName, pwchResource);
}

CResource::~CResource()
{
	delete [] m_pResourceName;
}

CParseItem::CParseItem(parseState ParseState)
{
	m_ParseState = ParseState;
}

CParseItem::~CParseItem()
{

}

CRDFContentHandler::CRDFContentHandler(ISAXXMLReader* pRdr)
{
	idnt = 0;
	m_parseStateCurrent = CParseItem::Ready;
	m_pRdr = pRdr;
	*m_pwchString = '\0';
}

CRDFContentHandler::~CRDFContentHandler()
{

}
        
HRESULT STDMETHODCALLTYPE CRDFContentHandler::characters( 
            /* [in] */ wchar_t __RPC_FAR *pwchChars,
            /* [in] */ int cchChars)
{
	HRESULT hr = S_OK;

	switch (m_parseStateCurrent)
	{
	/*
	 * [TODO] Other states will generally throw errors
	 */

	case CParseItem::PropertyElt:
		{
			/*
			 * Since SAX2 is non-validating we don't get a separate 'ignorable whitespace'
			 * event. This is because SAX can't tell whether an element is mixed content
			 * or element only. This gives us a problem in situations where whitespace has
			 * been used for layout purposes, like this:
			 *
			 *	<x>
			 *		<y>some text</y>
			 *	</x>
			 *
			 * We will get three character events:
			 *	- between the x and y opening elements
			 *	- between the y opening and closing elements
			 *	- between the y and x closing elements
			 *
			 * We shouldn't simply ignore all whitespace though, since to skip whitespace would mean
			 * we miss this:
			 *
			 *	<x>   a padded string    </x>
			 *
			 * We therefore ignore any content which is ONLY whitespace (by leaving the state as
			 * propertyElt), but any content that contains whitespace AND other text, we take the
			 * whole lot (and set the state to value)
			 */

			int i = cchChars;
			while (i--)
			{
				if (!isspace(pwchChars[i]))
				{
					swprintf(m_pwchString, L"%.*s", cchChars, pwchChars);

					/*
					 * Now push the state we are in onto the state stack, and set the current state
					 */

					CParseItem *pParseItem = new CParseItem(m_parseStateCurrent);
					m_ParseItemStack.Push(pParseItem);
					m_parseStateCurrent = CParseItem::String;
					break;
				}
			}
			break;
		}

	case CParseItem::String:

		{
			/*
			 * If we get more characters after we have had a string then we step over them. This
			 * would happen in a situation like this:
			 *
			 *	<x>
			 *		Nicely laid out for easy reading
			 *	</x>
			 */

			break;
		}

	default:
		break;
	}
    return hr;
}

HRESULT STDMETHODCALLTYPE CRDFContentHandler::startElement( 
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ wchar_t __RPC_FAR *pwchRawName,
            /* [in] */ int cchRawName,
            /* [in] */ ISAXAttributes __RPC_FAR *pAttributes)
{
	HRESULT hr = S_OK;
	BOOL bInRDF = !wcsncmp(pwchNamespaceUri, RDF_NS, cchNamespaceUri),
		 bGotDescription = false;
	CParseItem::parseState parseStateNew = m_parseStateCurrent;
	int cchResource;
	//SAXAttributes mxa;

	//mxa.setAttributes(pAttributes);

	/*
	 * Switch using the current parsing state
	 */

	switch (m_parseStateCurrent)
	{

	/*
	 * We begin in the Ready state. From here we check for the RDF tag or an obj (a description,
	 * a typedNode or a container)
	 */

	case CParseItem::Ready:
		if (bInRDF)
		{
			if (!wcsncmp(pwchLocalName, RDF_RDF, cchLocalName))
				parseStateNew = CParseItem::RDF;
			else if (!wcsncmp(pwchLocalName, RDF_DESCRIPTION, cchLocalName))
			{

				/*
				 * With a description we need to push the resource URI onto the subject stack,
				 * since all statements that follow at this level of nesting will be about
				 * this resource
				 */

				parseStateNew = CParseItem::Description;
				hr = getResource(pAttributes, m_pwchSubject, &cchResource, bInRDF);
				CResource *pResource = new CResource(m_pwchSubject);
				m_SubjectStack.Push(pResource);
			}
			else
			{
				wprintf(L"\nErr: RDF tag '%.*s' is invalid at top-level", cchLocalName, pwchLocalName);
				hr = E_FAIL;
			}
		}
		else
		{
			/*
			 * Any element that we have at this level is a typed node. As with a description
			 * we need to save the resource URI to the stack for the statements that are about
			 * to follow
			 */

			parseStateNew = CParseItem::TypedNode;
			hr = getResource(pAttributes, m_pwchSubject, &cchResource, bInRDF);
			CResource *pResource = new CResource(m_pwchSubject);
			m_SubjectStack.Push(pResource);
		}
		break;

	/*
	 * If we started with an RDF tag, then we only allow an obj
	 */

	case CParseItem::RDF:
		if (bInRDF)
		{
			if (!wcsncmp(pwchLocalName, RDF_DESCRIPTION, cchLocalName))
			{
				parseStateNew = CParseItem::Description;
				hr = getResource(pAttributes, m_pwchSubject, &cchResource, bInRDF);
				CResource *pResource = new CResource(m_pwchSubject);
				m_SubjectStack.Push(pResource);
			}
			else
			{
				wprintf(L"\nErr: RDF tag '%.*s' is invalid inside rdf:RDF", cchLocalName, pwchLocalName);
				hr = E_FAIL;
			}
		}
		else
		{
			parseStateNew = CParseItem::TypedNode;
			hr = getResource(pAttributes, m_pwchSubject, &cchResource, bInRDF);
			CResource *pResource = new CResource(m_pwchSubject);
			m_SubjectStack.Push(pResource);
		}
		break;

	case CParseItem::TypedNode:
	case CParseItem::Description:

		/*
		 * An element inside a description is a propertyElt. The element namespace followed
		 * by the local name is the predicate, and the value contained by the element is the
		 * object. Note that the syntax doesn't explicitly prevent elements from the RDF
		 * namespace, but here we only allow rdf:type
		 */

		if (!bInRDF)
		{
			swprintf(m_pwchPredicate, L"[%s%.*s]", pwchNamespaceUri, cchLocalName, pwchLocalName);
			CResource *pResource = new CResource(m_pwchPredicate);
			m_PredicateStack.Push(pResource);
			parseStateNew = CParseItem::PropertyElt;
		}
		else
		{
			if (!wcsncmp(pwchLocalName, RDF_TYPE, cchLocalName))
			{
				swprintf(m_pwchPredicate, L"[%s%.*s]", pwchNamespaceUri, cchLocalName, pwchLocalName);
				CResource *pResource = new CResource(m_pwchPredicate);
				m_PredicateStack.Push(pResource);
				parseStateNew = CParseItem::Type;

				/*
				 * [TODO] Need to get the object from the resource attribute
				 */
			}
			else
			{
				wprintf(L"\npropertyElt '%.*s' is invalid since it is from the RDF namespace", cchLocalName, pwchLocalName);
				hr = E_FAIL;
			}
		}
		break;

	/*
	 * Inside a propertyElt we can have an obj or a string. If the type of propertyElt has been
	 * modified, we may have a parseLiteral or a parseResource, but they are handled seperately
	 */

	case CParseItem::PropertyElt:
		{
			if (bInRDF)
			{
				if (!wcsncmp(pwchLocalName, RDF_DESCRIPTION, cchLocalName))
					parseStateNew = CParseItem::Description;
				else
				{
					wprintf(L"\nErr: RDF tag '%.*s' is invalid inside a propertyElt", cchLocalName, pwchLocalName);
					hr = E_FAIL;
					break;
				}
			}
			else
				parseStateNew = CParseItem::TypedNode;

			/*
			 * Once we get the resource URI of the obj we have enough information to make a
			 * statement
			 */

			hr = getResource(pAttributes, m_pwchString, &cchResource, bInRDF);
			CResource *r = m_SubjectStack.Peek();
			swprintf(m_pwchSubject, L"%s", r->m_pResourceName);
			r = m_PredicateStack.Peek();
			swprintf(m_pwchPredicate, L"%s", r->m_pResourceName);
			wprintf(L"\n\nSubj->%s", m_pwchSubject);
			wprintf(L"\nPred->%s", m_pwchPredicate);
			wprintf(L"\nObj-->%s", m_pwchString);

			/*
			 * Now we can push the resource for the next set of statements
			 */

			CResource *pResource = new CResource(m_pwchString);
			m_SubjectStack.Push(pResource);
		}
		break;

	case CParseItem::String:
		wprintf(L"\nUnexpected element '%.*s'. string can only contain string literal as value", cchLocalName, pwchLocalName);
		hr = E_FAIL;
		break;

	default:
		break;
	}

	switch (parseStateNew)
	{
	case CParseItem::TypedNode:
		wprintf(L"\n\nSubj->%s", m_pwchSubject);
		wprintf(L"\nPred->[%s%s]", RDF_NS, RDF_TYPEATTR);
		wprintf(L"\nObj-->[%s%s]", pwchNamespaceUri, pwchLocalName);

		/* Drop through */

	case CParseItem::Description:

		/*
		 * Now we run through the attributes again, this time looking for non-RDF attributes
		 */

		int l;
		wchar_t *ns, *ln, *qn, *v;
		int ins, iln, iqn, iv;
		pAttributes->getLength(&l);
		for (int i = 0; i < l; i++)
		{
			pAttributes->getName(i, &ns, &ins, &ln, &iln, &qn, &iqn);
			pAttributes->getValue(i, &v, &iv);

			/*
			 * XML namespace attributes don't show up, so watch for them
			 */

			if (ins || iln)
			{

				/*
				 * If the attribute we are processing is in the RDF namespace, or there is no namespace
				 * but the current element is in the RDF namespace, then we can check for RDF attributes
				 */

				if ((!wcsncmp(ns, RDF_NS, ins) || (!ins && bInRDF)))
				{
				
					/*
					 * The next attribute indicates the type of the resource
					 */

					if (!wcsncmp(ln, RDF_TYPEATTR, __max((int) wcslen(RDF_TYPEATTR), iln)))
					{
						wprintf(L"\n\nSubj->%s", m_pwchSubject);
						wprintf(L"\nPred->[%s%s]", RDF_NS, RDF_TYPEATTR);
						wprintf(L"\nObj-->[%s]", v, iv);
					}
				}
				else
				{
					wprintf(L"\n\nSubj->%s", m_pwchSubject);
					wprintf(L"\nPred->[%s%.*s]", ns, iln, ln);
					wprintf(L"\nObj-->\"%s\"", v);
				}
			} // if ( either the namespace or local name are set )
		}
		break;
	}

	/*
	 * Now push the state we are in onto the state stack, and set the current state to the state
	 * that we have just determined
	 */

	CParseItem *pParseItem = new CParseItem(m_parseStateCurrent);
	m_ParseItemStack.Push(pParseItem);
	m_parseStateCurrent = parseStateNew;
    return hr;
}

HRESULT CRDFContentHandler::getResource(
			ISAXAttributes __RPC_FAR *pAttributes,
			wchar_t *m_pwchSubject,
			int *cchResource,
			BOOL bInRDF)
{
	HRESULT hr = S_OK;

	/*
	 * Run through the attributes to find RDF specific ones:
	 *
	 *	about
	 *	ID
	 *	aboutEach
	 *	aboutEachPrefix
	 *	bagID
	 *	type
	 *	parseType
	 *	resource
	 */

	int l;
	wchar_t *ns, *ln, *qn, *v;
	int ins, iln, iqn, iv;
	BOOL bGotidAboutAttr = false;

	pAttributes->getLength(&l);
	for (int i = 0; i < l; i++)
	{
		pAttributes->getName(i, &ns, &ins, &ln, &iln, &qn, &iqn);

		/*
		 * XML namespace attributes don't show up, so watch for them
		 */

		if (ins || iln)
		{
			pAttributes->getValue(i, &v, &iv);

			/*
			 * If the attribute we are processing is in the RDF namespace, or there is no namespace
			 * but the current element is in the RDF namespace, then we can check for RDF attributes
			 */

			if ((!wcsncmp(ns, RDF_NS, ins) || (!ins && bInRDF)))
			{

				/*
				 * The first set of attributes set the resource being described
				 */

				if (!wcsncmp(ln, RDF_IDATTR, __max((int) wcslen(RDF_IDATTR), iln)))
				{
					if (bGotidAboutAttr)
					{
						printf("\nOnly zero or one idAboutAttr allowed");
						hr = E_FAIL;
					}
					wchar_t *pwchBaseURL;
					m_pRdr->getBaseURL(&pwchBaseURL);
					*cchResource = swprintf(m_pwchSubject, L"[%s#%s]", pwchBaseURL, v);
					bGotidAboutAttr = true;
				}

				if (!wcsncmp(ln, RDF_ABOUTATTR, __max((int) wcslen(RDF_ABOUTATTR), iln)))
				{
					if (bGotidAboutAttr)
					{
						printf("\nOnly zero or one idAboutAttr allowed");
						hr = E_FAIL;
					}
					*cchResource = swprintf(m_pwchSubject, L"[%s]", v);
					bGotidAboutAttr = true;
				}

				if (!wcsncmp(ln, RDF_ABOUTEACHATTR, __max((int) wcslen(RDF_ABOUTEACHATTR), iln)))
				{
					if (bGotidAboutAttr)
					{
						printf("\nOnly zero or one idAboutAttr allowed");
						hr = E_FAIL;
					}
					*cchResource = swprintf(m_pwchSubject, L"distributant([%s])", v);
					bGotidAboutAttr = true;
				}

				if (!wcsncmp(ln, RDF_ABOUTEACHPREFIXATTR, __max((int) wcslen(RDF_ABOUTEACHPREFIXATTR), iln)))
				{
					if (bGotidAboutAttr)
					{
						printf("\nOnly zero or one idAboutAttr allowed");
						hr = E_FAIL;
					}
					*cchResource = swprintf(m_pwchSubject, L"distributant([%s])", v);
					bGotidAboutAttr = true;
				}
			}
		} // if ( either the namespace or local name are set )
	} // for ( each attribute on this element )

	/*
	 * If we have no resource URI yet, then set an anonymous one
	 */

	if (!bGotidAboutAttr)
	{
		*cchResource = swprintf(m_pwchSubject, L"anon");
	}
	return hr;
}
       
HRESULT STDMETHODCALLTYPE CRDFContentHandler::endElement( 
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ wchar_t __RPC_FAR *pwchRawName,
            /* [in] */ int cchRawName)
{
	/*
	 * Closing an element may involve closing off more than one state. For example, say we had:
	 *
	 *	<rdf:Description>
	 *		<x:y>text</x:y>
	 *	</rdf:Description>
	 *
	 * When we see the closing x:y element, we don't just close off the string, but we close off
	 * the propertyElt state too. Since other situations may arise where this is necessary we
	 * just create a loop
	 */

	BOOL bRepeat = false;

	do
	{
		bRepeat = false;
		switch (m_parseStateCurrent)
		{

		/*
		 * A description or typedNode will have saved a resource URI to the stack, so pop
		 * it off
		 */

		case CParseItem::Description:
			{
				CResource *r = m_SubjectStack.Pop();
				wprintf(L"\nDESC Pop: %s", r->m_pResourceName);
				delete r;
			}
			break;

		case CParseItem::TypedNode:
			{
				CResource *r = m_SubjectStack.Pop();
				wprintf(L"\nTN Pop: %s", r->m_pResourceName);
				delete r;
			}
			break;

		/*
		 * A propertyElt would have saved a predicate URI to the stack, so pop it off
		 */

		case CParseItem::PropertyElt:
			{
				CResource *r = m_PredicateStack.Pop();
				wprintf(L"\nPE Pop: %s", r->m_pResourceName);
				delete r;
			}
			break;

		/*
		 * If we are closing a string then we now have enough information to make a statement.
		 * Note that by only peeking at the predicate, rather than popping it, we create the
		 * possibility for multiple nested statements5
		 */

		case CParseItem::String:
		{
			CResource *r = m_SubjectStack.Peek();
			swprintf(m_pwchSubject, L"%s", r->m_pResourceName);
			r = m_PredicateStack.Peek();
			swprintf(m_pwchPredicate, L"%s", r->m_pResourceName);
			wprintf(L"\n\nSubj->%s", m_pwchSubject);
			wprintf(L"\nPred->%s", m_pwchPredicate);
			wprintf(L"\nObj-->\"%s\"", m_pwchString);

			/*
			 * Now that we are finished with the string clear it for next time
			 */

			*m_pwchString = '\0';

			/*
			 * Since we have just closed out a string then we need to do another pop to close
			 * out the element, because strings are not nested
			 */

			bRepeat = true;
		}

		default:
			break;
		}

		/*
		 * As we close off each element, return to the previous state
		 */

		CParseItem *pParseItem = m_ParseItemStack.Pop();
		m_parseStateCurrent = pParseItem->m_ParseState;
		delete pParseItem;
	}
	while (bRepeat);

    return S_OK;
}

HRESULT STDMETHODCALLTYPE CRDFContentHandler::startDocument()
{
    printf("<?xml version=\"1.0\" ?>");
    return S_OK;
}
        
void CRDFContentHandler::prt(
            /* [in] */ const wchar_t * pwchFmt,
            /* [in] */ const wchar_t __RPC_FAR *pwchVal,
            /* [in] */ int cchVal)
{
    static wchar_t val[1000];
	cchVal = cchVal>999 ? 999 : cchVal;
    wcsncpy( val, pwchVal, cchVal ); val[cchVal] = 0;
    wprintf(pwchFmt,val);
}

