// RDFContentHandler.h: interface for the RDFContentHandler class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RDFContentHandler_H__E1B3AF99_0FA6_44CD_82E3_55719F9E3806__INCLUDED_)
#define AFX_RDFContentHandler_H__E1B3AF99_0FA6_44CD_82E3_55719F9E3806__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SAXContentHandler.h"

#define	RDF_NS			L"http://www.w3.org/1999/02/22-rdf-syntax-ns#"

#define	RDF_RDF			L"RDF"
#define	RDF_DESCRIPTION	L"Description"
#define	RDF_TYPE		L"type"

#define	RDF_IDATTR				L"ID"
#define	RDF_ABOUTATTR			L"about"
#define	RDF_ABOUTEACHATTR		L"aboutEach"
#define	RDF_ABOUTEACHPREFIXATTR	L"aboutEachPrefix"

#define	RDF_TYPEATTR			L"type"

class CResource : public CObject
{
public:
    CResource(const wchar_t * pwchResourceName);
    virtual ~CResource();

public:
	wchar_t *m_pResourceName;
};

class CResourceStack : public CTypedPtrList< CObList, CResource* >
{
public:
	virtual ~CResourceStack();

    // Add element to top of stack
    void Push( CResource* newResource )
        { AddHead( newResource ); }

    // Peek at top element of stack
    CResource* Peek()
        { return IsEmpty() ? NULL : GetHead(); }

    // Pop top element off stack
    CResource* Pop()
        { return RemoveHead(); }
};

class CParseItem : public CObject
{
public:
	enum parseState
	{
		Finished = 0,
			Ready,
			RDF,
			Description,
			PropertyElt,
			Type,
			TypedNode,
			Value,
			String
	};
    CParseItem(parseState ParseState);
    virtual ~CParseItem();

public:
	parseState m_ParseState;
};

class CParseItemStack : public CTypedPtrList< CObList, CParseItem* >
{
public:
	virtual ~CParseItemStack();

    // Add element to top of stack
    void Push( CParseItem* newParseItem )
        { AddHead( newParseItem ); }

    // Peek at top element of stack
    CParseItem* Peek()
        { return IsEmpty() ? NULL : GetHead(); }

    // Pop top element off stack
    CParseItem* Pop()
        { return RemoveHead(); }
};

class CElement : public CObject { };

class CElementStack : public CTypedPtrList< CObList, CElement* >
{
public:
	virtual ~CElementStack();

    // Add element to top of stack
    void Push( CElement* newElement )
        { AddHead( newElement ); }

    // Peek at top element of stack
    CElement* Peek()
        { return IsEmpty() ? NULL : GetHead(); }

    // Pop top element off stack
    CElement* Pop()
        { return RemoveHead(); }
};

class CRDFContentHandler : public CSAXContentHandler
{
public:
    CRDFContentHandler(ISAXXMLReader* pRdr);
    virtual ~CRDFContentHandler();

        virtual HRESULT STDMETHODCALLTYPE characters( 
            /* [in] */ wchar_t __RPC_FAR *pwchChars,
            /* [in] */ int cchChars);

        virtual HRESULT STDMETHODCALLTYPE startElement( 
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ wchar_t __RPC_FAR *pwchRawName,
            /* [in] */ int cchRawName,
            /* [in] */ ISAXAttributes __RPC_FAR *pAttributes);
        
        virtual HRESULT STDMETHODCALLTYPE endElement( 
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ wchar_t __RPC_FAR *pwchRawName,
            /* [in] */ int cchRawName);

        virtual HRESULT STDMETHODCALLTYPE startDocument();

		virtual HRESULT getResource(
			ISAXAttributes __RPC_FAR *pAttributes,
			wchar_t *pwchResource,
			int *pcchResource,
			BOOL bInRDF);

private:
        void prt(
            /* [in] */ const wchar_t * pwchFmt,
            /* [in] */ const wchar_t __RPC_FAR *pwchVal,
            /* [in] */ int cchVal);
		int idnt;
		ISAXXMLReader* m_pRdr;
		CResourceStack m_SubjectStack,
					   m_PredicateStack;
		CParseItemStack m_ParseItemStack;
		CParseItem::parseState m_parseStateCurrent;
		wchar_t m_pwchSubject[1000],
				m_pwchPredicate[1000],
				m_pwchString[1000];
};

#endif // !defined(AFX_RDFContentHandler_H__E1B3AF99_0FA6_44CD_82E3_55719F9E3806__INCLUDED_)
