// RDFXMLParser.cpp: implementation of the CRDFXMLParser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Impl.h"
#include "RDFXMLParser.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRDFXMLParser::CRDFXMLParser()
{
	m_parseStateCurrent = CParseItem::Ready;
}

CRDFXMLParser::~CRDFXMLParser()
{

}


CElement* CRDFXMLParser::createElement(void)
{
	return new CRDFElement();
}


HRESULT STDMETHODCALLTYPE CRDFXMLParser::startElement(
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ wchar_t __RPC_FAR *pwchRawName,
            /* [in] */ int cchRawName,
            /* [in] */ ISAXAttributes __RPC_FAR *pAttributes)
{
	_startElement(pwchNamespaceUri, cchNamespaceUri, pwchLocalName, cchLocalName, pAttributes);

	HRESULT hr = S_OK;
	CQName *pName = m_pCurrent->getName();
	CParseItem::parseState parseStateNew = m_parseStateCurrent;

	/*
	 * Switch using the current parsing state
	 */

	switch (m_parseStateCurrent)
	{

	/*
	 * We begin in the Ready state. From here we check for the RDF tag or an obj (a description,
	 * a typedNode or a container)
	 */

		case CParseItem::Ready:
			if (((CRDFElement*) m_pCurrent)->m_bInRDFNamespace)
			{
				if (pName->getLocalName() == RDF_RDF)
				{
					parseStateNew = CParseItem::RDF;
					m_pRDFConsumer->startModel();
				}
				else if (pName->getLocalName() == RDF_DESCRIPTION)
				{

					/*
					 * With a description we need to push the resource URI onto the subject stack,
					 * since all statements that follow at this level of nesting will be about
					 * this resource
					 */

					parseStateNew = CParseItem::Description;
					//hr = getResource(pAttributes, m_pwchSubject, &cchResource, bInRDF);
					//CResource *pResource = new CResource(m_pwchSubject);
					//m_SubjectStack.Push(pResource);
				}
				else
				{
					hr = fatalError((ISAXLocator *) NULL, L"\nErr: RDF tag '%.*s' is invalid at top-level", 0);
				}
			}
			break;

		default:
			parseStateNew = CParseItem::Unknown;
			break;
	}

	CQName qn(_T("about"));
	CString *a = m_pCurrent->getAttribute(&qn);
	delete a;

	//if (parseStateNew == CParseItem::Description)
	//	getResource();

	/*
	 * Now push the state we are in onto the state stack, and set the current state to the state
	 * that we have just determined
	 */

	CParseItem *pParseItem = new CParseItem(m_parseStateCurrent);
	m_ParseItemStack.Push(pParseItem);
	m_parseStateCurrent = parseStateNew;
    return hr;

	//wprintf(L"Start Element: %.*s\n", cchLocalName, pwchLocalName);
	//CResource *p = (CResource *) createResource(_T("dc:"), _T("Author"));
	//CResource *s = (CResource *) createResource(_T("http://www.spiked-oonline.com/"));
	//CLiteral *l = (CLiteral *) createLiteral(_T("Mick Hume"));

	//CStatement *st = (CStatement *) createStatement(s, p, l);

	//m_pRDFConsumer->addStatement(st);

	//delete st;
	//delete l;
	//delete s;
	//delete p;
    //return S_OK;
}

HRESULT STDMETHODCALLTYPE CRDFXMLParser::endElement( 
            /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
            /* [in] */ int cchNamespaceUri,
            /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
            /* [in] */ int cchLocalName,
            /* [in] */ wchar_t __RPC_FAR *pwchRawName,
            /* [in] */ int cchRawName)
{
	_endElement(pwchNamespaceUri, cchNamespaceUri, pwchLocalName, cchLocalName);

	HRESULT hr = S_OK;


	/*
	 * Act on the state
	 */

	switch (m_parseStateCurrent)
	{

		/*
		 * 
		 */

		case CParseItem::RDF:
			{
				m_pRDFConsumer->endModel();
			}
			break;

		default:
			break;
	}

	/*
	 * Pop the previous state back off the state stack
	 */

	CParseItem *pParseItem = m_ParseItemStack.Pop();
	m_parseStateCurrent = pParseItem->m_ParseState;
	delete pParseItem;
    return hr;
}

CRDFElement::CRDFElement()
{
	m_pAbout = NULL;
	m_bInRDFNamespace = false;
}

CRDFElement::~CRDFElement()
{
	delete m_pAbout;
}

void CRDFElement::putName(CQName *pName)
{
	m_bInRDFNamespace = (pName->getNamespace() == RDF_NS);
	CElement::putName(pName);
}

CParseItem::CParseItem(parseState ParseState)
{
	m_ParseState = ParseState;
}

CParseItem::~CParseItem()
{

}

CParseItemStack::~CParseItemStack()
{
	while (!this->IsEmpty())
	{
		delete this->GetHead();
		this->RemoveHead();
	}
}
