// RDFXMLParser.h: interface for the CRDFXMLParser class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RDFXMLPARSER_H__C6E44C0B_244E_4912_9EDA_B14E214C6745__INCLUDED_)
#define AFX_RDFXMLPARSER_H__C6E44C0B_244E_4912_9EDA_B14E214C6745__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "GenericXML2RDF.h"

#define	RDF_NS			L"http://www.w3.org/1999/02/22-rdf-syntax-ns#"

#define	RDF_RDF			L"RDF"
#define	RDF_DESCRIPTION	L"Description"
#define	RDF_TYPE		L"type"

#define	RDF_IDATTR				L"ID"
#define	RDF_ABOUTATTR			L"about"
#define	RDF_ABOUTEACHATTR		L"aboutEach"
#define	RDF_ABOUTEACHPREFIXATTR	L"aboutEachPrefix"

#define	RDF_TYPEATTR			L"type"

/*
 * States for parsing, and a stack to hold them in
 */

class CParseItem : public CObject
{
public:
	enum parseState
	{
		Unknown = 0,
		Ready,
		RDF,
		Description,
		PropertyElt,
		Type,
		TypedNode,
		Value,
		String
	};
    CParseItem(parseState ParseState);
    virtual ~CParseItem();

public:
	parseState m_ParseState;
};

class CParseItemStack : public CTypedPtrList< CObList, CParseItem* >
{
public:
	virtual ~CParseItemStack();

    // Add element to top of stack
    void Push( CParseItem* newParseItem )
        { AddHead( newParseItem ); }

    // Peek at top element of stack
    CParseItem* Peek()
        { return IsEmpty() ? NULL : GetHead(); }

    // Pop top element off stack
    CParseItem* Pop()
        { return RemoveHead(); }
};

class CRDFElement : public CElement
{
public:
	CRDFElement();
	virtual ~CRDFElement();

	bool m_bInRDFNamespace;

protected:
	CQName *m_pAbout;

/*
 * From CElement
 */

public:
	virtual void putName(CQName *pName);
	//virtual void putAttribute(CQName *pName, wchar_t *pstrValue, int cchValue);

};

class CRDFXMLParser : public CGenericXML2RDF  
{
public:
	CRDFXMLParser();
	virtual ~CRDFXMLParser();

protected:
	CParseItem::parseState m_parseStateCurrent;
	CParseItemStack m_ParseItemStack;

	/*
	 * From CGenericParser
	 */

public:
	virtual CElement* createElement(void);

	virtual HRESULT STDMETHODCALLTYPE startElement(
		/* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
		/* [in] */ int cchNamespaceUri,
		/* [in] */ wchar_t __RPC_FAR *pwchLocalName,
		/* [in] */ int cchLocalName,
		/* [in] */ wchar_t __RPC_FAR *pwchRawName,
		/* [in] */ int cchRawName,
		/* [in] */ ISAXAttributes __RPC_FAR *pAttributes);

	virtual HRESULT STDMETHODCALLTYPE CRDFXMLParser::endElement( 
        /* [in] */ wchar_t __RPC_FAR *pwchNamespaceUri,
        /* [in] */ int cchNamespaceUri,
        /* [in] */ wchar_t __RPC_FAR *pwchLocalName,
        /* [in] */ int cchLocalName,
        /* [in] */ wchar_t __RPC_FAR *pwchRawName,
        /* [in] */ int cchRawName);
};

#endif // !defined(AFX_RDFXMLPARSER_H__C6E44C0B_244E_4912_9EDA_B14E214C6745__INCLUDED_)
