import org.w3c.dom.*;
import org.xml.sax.*;
import java.util.*;

/**
 *  Demonstration program for meaning-level Java API to XML
 *  Takes 2 arguments:
 *     (1) an XML filename
 *     (2) filename for the MDL definition of that XML
 *  Runs the application code in method application() against the XML,
 *  using the MDL to get the required inforoamtion from the XML.
 */
public class collApp
{
  protected String XMLFileName = null;
  protected String MDLFileName = null;
  
  protected MDL mdl = null;
  protected XMLIO xio;
  protected XFactory xf;  

  /** Process input args and open the XML file  */
  public void doMain(String[] args)
  {

    XMLFileName = args[0];
    MDLFileName = args[1];

    if ((XMLFileName != null) && (XMLFileName.length() > 0)
        && (MDLFileName != null) && (MDLFileName.length() > 0))
    {
      
      //   open MDL file and process it
      mdl = new MDL(MDLFileName,false); // no detailed trace info
      mdl.captureMDL();
      message("MDL read in;");
      
      if (mdl.valid())
      {
        // read XML input file
        xio = new XMLIO(false);
        Element root = xio.readXMLFile(XMLFileName);
        message("XML file read in;");
        
        // create factory for objects represented in the XML
        xf = new XFactory(root,mdl);
        
        // run the application code
        application(xf);
      }
      else {message("Application not run because of MDL errors.");}
    }
    else
    {
      message("Bad input args: " + XMLFileName + ", " + MDLFileName);
    }
  }
  
  /** this is the actual application code, written in terms of application domain
  classes. */
  void application(XFactory xf)
  {
    int i,j;
    course c;
    teacher t;
    
    // retrieve courses and then get their teachers
    message("COURSES OUTER LOOP");
    Vector courses = xf.every_course();
    for (i = 0; i < courses.size(); i++)
    {
        c = (course)courses.elementAt(i);
        t = c.teacher_teaches();
        message("Course: " + c.name() + " Teacher: " + t.surName());
    }
    
    // retrieve teachers and then get their courses
    message("TEACHERS OUTER LOOP");
    Vector teachers = xf.every_teacher();
    for (i = 0; i < teachers.size(); i++)
    {
        t = (teacher)teachers.elementAt(i);
        Vector ct = t.teaches_course();
        for (j = 0; j < ct.size(); j++)
        {
            c = (course)ct.elementAt(j);
            message("Course: " + c.name() + " Teacher: " + t.surName());
        }
    }
  }
    
  /** Main method to run from the command line.    */
  public static void main (String[] args)
  {
    if (args.length != 2)
    {
      System.out.println("java collApp MDLFileName XMLFileName"
                         + " runs the application collApp"
                         + " against XMLFileName"
                         + " whose MDL is in MDLFileName.");
      return;
    }
    collApp app = new collApp();
    app.doMain(args);
  }

void message(String s) {System.out.println(s);}


}