import java.io.*;
import java.util.*;

/**
 *  Generates Java source for MDL access to a DAML model
 *  Takes 1 argument:
 *     (1) a DAML model filename
 */
public class MGen
{
    private DAML daml;
    protected String DAMLFileName = null;
    protected String SelectFileName = null;
    Vector selectedClasses; // names of classes you want to generate code for
    private classWriter cl;

  /** Process input args and open the DAML file  */
  public void doMain(String[] args)
  {
    String selectString;
    DAMLFileName = args[0];
    SelectFileName = args[1];

    if ((DAMLFileName != null) && (DAMLFileName.length() > 0)
        && (SelectFileName != null) && (SelectFileName.length() > 0))
    {
        message("Reading DAML model from " + DAMLFileName);
        daml = new DAML(DAMLFileName,true);
        daml.captureDAML();
        
        selectString = readQuery(SelectFileName);
        selectedClasses = stripWords(selectString);
        
        cl = new classWriter(daml,selectedClasses);
        cl.writeAllSource();
    }
    else
    {
      message("Bad input args: " + DAMLFileName);
    }
  }

  /** Main method to run from the command line.    */
  public static void main (String[] args)
  {
    if (args.length != 2)
    {
      System.out.println("java MGen DAMLFileName SelectFileName"
                         + " generates Java source files for MDL access to classes in a DAML model"
                         + " using only classes named in SelectFileName.");
      return;
    }
    MGen app = new MGen();
    app.doMain(args);
  }
  
//------------------------------------------------------------------------------------
//           Read in and process a text file of selected class names
//------------------------------------------------------------------------------------
  
  
  // read in a text file and remove new lines from it.
  String readQuery(String fileName)
  {
    byte[] bytes = new byte[4000]; // maximum query length
    byte[] nBytes = new byte[4000]; 
    int i,j;
    String res = null;
    try{
        FileInputStream fi = new FileInputStream(fileName);
        try {
            i = fi.read(bytes);
            nBytes = oneLine(bytes,i);
            res = new String(nBytes,0,i);
        }
        catch (IOException e) {message("IO exception: " +e.getMessage());}
    }
    catch (FileNotFoundException e) {message("Query file not found.");}
    return res;
  }
  
  /* remove all newlines from a byte array of length len,
  replacing them by double spaces */
  byte[] oneLine(byte[] in, int len)
  {
    int i;
    byte[] res = new byte[4000];
    i=0; // position in input and result byte arrays
    while  (i < len) 
    {
        // replace any 'carriage return line feed' by two spaces
        if ((i < len - 1) && (in[i] == 13) && (in[i+1] == 10)) 
            {res[i] = ' '; res[i+1] = ' '; i = i+2;}
        // move across any other character unchanged
        else {res[i] = in[i]; i++;}
    }
    return res;
  }

      
     /** from a string s, strip out words separated by spaces or commas. */
     private Vector stripWords(String s)
     {
        String sTemp; // disposable copy
        String word;
        char c;
        Vector res = new Vector();
        
        sTemp = s.substring(0); // make a straight copy of s
        while (sTemp.length() > 0)
        {
            c = sTemp.charAt(0);
            if ((c == ' ')|(c == ','))
            {
                sTemp = sTemp.substring(1); // strip off the first space
            }
            else if ((c == ''')|(c == '"'))
            {
                word = stringBetween(sTemp,c); // string between quotes
                sTemp = sTemp.substring(word.length()+2); //strip off string and quotes
                res.addElement(word);
            }
            else
            {
                word = firstWord(sTemp);
                sTemp = sTemp.substring(word.length()); // strip off word
                res.addElement(word);
            }
        }
        return res;
     }
     
          
     /* the first character of s is c. Find the string between that and the
     next occurrence of c, or the end of s. */
     private String stringBetween(String s, char c)
     {
        int i;
        char cp;
        String res = s.substring(1); // in case no repeat of c is found
        boolean found = false; // no repeat of c found yet
        
        for (i = 1; i < s.length(); i++) // don't look for c in 1st char
        {
            cp = s.charAt(i);
            if ((!found) && (c == cp)) // first repeat of c only
                {found = true; res = s.substring(1,i);}
        }
        return res;
     }

	          
     /** find the first word in s before any space or comma. 
     The first character of s must not be a space or comma. */
     private String firstWord(String s)
     {
        int i;
        char c;
        String res = s; // in case no blank is found
        boolean found = false; // no blank found yet
        
        for (i = 1; i < s.length(); i++) // don't look for blank in 1st char
        {
            c = s.charAt(i);
            if ((!found) && ((c == ' ')|(c == ','))) // first blank or comma only
                {found = true; res = s.substring(0,i);}
        }
        return res;
     }


void message(String s) {System.out.println(s);}

  
} // end of class MGen

