import java.io.*;
import java.util.*;
import org.apache.xerces.parsers.*;
import org.apache.xerces.dom.*;
import org.apache.xml.serialize.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/** superclass for anything that needs to read or write XML,
using the Apache Xerces parser */

public class XMLIO
{
	DocumentImpl outDoc; // XML document to be output
    FileOutputStream fos;
	Element topOut; // top element of output MD:
	boolean testing; // true for diagnostic writes of a lot of stuff
    
    public XMLIO(boolean test)
    {
        testing = test;
    }
    
        
//--------------------------------------------------------------------------------------
//                              Output File handling
//--------------------------------------------------------------------------------------
    
        
    void makeOutputDocument(String fileName)
    {
	    fos = getOutputStream(fileName);
	    message("Creating document;");
	    outDoc = new DocumentImpl();
    }

    FileOutputStream getOutputStream(String fileName)
    {
        File fo;        
	    FileOutputStream fos = null;
	    fo = new File(fileName);
	    try {
	        fos = new FileOutputStream(fo);
	    }
	    catch (java.io.IOException e) {message("Output file opening exception: " + e.getMessage());}
	    return fos;
    }
    
    void writeXMLOutput(FileOutputStream fos, Document doc)
    {
        XMLSerializer XSer;
        OutputFormat of;

	    of = new OutputFormat("xml","UTF-8",true);
	    XSer = new XMLSerializer(fos,of);
	    try{
	        XSer.serialize(doc);
	        message("Output serialised");
	        fos.close();
	    }
	    catch (java.io.IOException e) {message("XML serialising exception");}
	    message("Closed.");
    }
    
        
//--------------------------------------------------------------------------------------
//                              Input File handling
//--------------------------------------------------------------------------------------

    /** 
	* Open an XML file and return its root element 
	*/
	Element readXMLFile(String location)
	{
	    FileInputStream fi = null; // the actual thing you read from
        InputSource inSource; // form used by Apache XML parser   
        DOMParser XercesParse;
	    Document doc;        
	    Element root= null;

	    tmessage("readXMLFile: " + location);
	    try {
	        fi = new FileInputStream(location);
	        inSource = new InputSource(fi);
	        XercesParse = new DOMParser();
	        XercesParse.parse(inSource);
	        doc = XercesParse.getDocument();
	        root = doc.getDocumentElement();
	        if (root == null) 
	            {message("No root element.");}
	        else
	            {
	                tmessage("File read from " + location);
	            }
	    }
	    catch (FileNotFoundException ex) {message("Cannot find file " + location);}
	    catch (java.io.IOException e) 
	        {message("Exception opening file " + location);}
	    catch (org.xml.sax.SAXException e) 
	        {message("SAX exception opening file " 
	            + location + ": " + e.getMessage());}
	    catch (Exception e) 
	        {message("Unidentified exception opening file " 
	            + location + ": " + e.getMessage());}
	    
	    return root;
    }
    
        
//--------------------------------------------------------------------------------------
//                              XML Utilities
//--------------------------------------------------------------------------------------
    
    // create an empty element in a namespace
    ElementNSImpl NSElement(String prefix, String name, String URI)
    {
        return (ElementNSImpl) outDoc.createElementNS(URI, prefix + ":" + name);
    }
    
    // create an element in a namespace, with text content
    ElementNSImpl textNSElement(String prefix, String name, String URI, String text)
    {
        TextImpl t;
        ElementNSImpl e = NSElement( prefix,  name,  URI);
        t= new TextImpl(outDoc,text);
        e.appendChild(t);
        return e;
    }
 
            
    ElementImpl newElement(String name)
    {
        return (ElementImpl) outDoc.createElement(name);
    }

/** when element el can have at most one child element with local name cName,
    return that element, or null if there is none.  
    Write a warning if there is more than one. */
    Element singleChild(Element el,String cName)
    {
        Element res = null;
        Vector v = childElements(el,cName);
        if (v.size() > 0)
        {
            if (v.size() == 1) {res = (Element)v.elementAt(0);}
            else message("Warning: element has " + v.size() + " child elements with local name " + cName);
        }
        return res;
    }

    
    
    /** Return a vector of all child elements of an element with a given local name */
    Vector childElements(Element el,String cName)
    {
        NodeList nodes;
        Node nd;
        int i;
        Element eg;
        Vector res;

        res = new Vector();
        nodes = el.getChildNodes();
        for (i = 0; i < nodes.getLength(); i++)
        {
            nd = nodes.item(i);
            if (nd instanceof Element)
            {
                eg = (Element)nd;
                if (getLocName(eg).equals(cName))
                    {res.addElement(eg);}
            }
        }
        return res;
    }
    
    /** return the text value of a node, which may be an Element or attribute */
    public String textValue(Node n)
    {
        String res = null;
        if (n == null) {message("Text-holding element or attribute node is null.");}
        else
        {
            if (n instanceof Element) {res = getText((Element)n);}
            else if (n instanceof Attr) {res = ((Attr)n).getValue();}
            else {
                message("Node is neither an element or an attribute. ");
                message("Class of node: " + n.getClass().getName());
            }
        }
        
        return res;
    }
    
    
    /* get the single text string in an element, or "" if there is none */
    public String getText(Element el)
    {
        NodeList nodes;
        Node nd;
        int i;
        String res = "";
        boolean found;
        TextImpl te;
        
        found = false;
        nodes = el.getChildNodes();
        for (i = 0; i < nodes.getLength(); i++)
        {
            nd = nodes.item(i);
            if (nd instanceof Text)
            {
                te = (TextImpl)nd;
                try
                {
                    res = te.getData();
                    if (found) {message("More than one child text node .");}
                    found = true;
                }
                catch (Exception e) {message(e.getMessage());} 
            }
        }
        return res;
    }


    /** add a comment below node el */
    public void addComment(Element el, String cText)
    {
	    Comment com = outDoc.createComment(cText);
        el.appendChild(com);
    }
    
    	
	String getLocName(Element el)
	{
	    String name = "not found";
	    try{
	        ElementNSImpl ne = (ElementNSImpl) el;
	        name = ne.getLocalName();
	    }
	    catch (Exception ex) {message("Apache exception getting local name." + ex.getMessage());}
	    return name;
	}
	
	// convert a NodeList to a Vector of Nodes; return null if the nodeList is null
	Vector nodeVector(NodeList nl)
	{
	    int i;
	    Vector res = null;
	    Node n;
	    if (nl != null)
	    {
	        res = new Vector();
            for (i = 0; i < nl.getLength(); i++)
            {
                n = (Node)nl.item(i);
                if (n == null) {message("Null node in Nodelist from XPath");}
                else {res.addElement(n);}
            }
	    }
	    return res;
	}
	
	/* check that an element only has child elements with given local names */
	public boolean checkChildElements(Element el,String[] allowed)
	{
	    boolean res = true;
        NodeList nodes;
        Node nd;
        int i;
        Element eg;
        String pName,eName;

        pName = getLocName(el);
        nodes = el.getChildNodes();
        for (i = 0; i < nodes.getLength(); i++)
        {
            nd = nodes.item(i);
            if (nd instanceof Element)
            {
                eg = (Element)nd;
                eName = getLocName(eg);
                if (!inArray(eName,allowed))
                {
                    message("Unexpected element '" + eName + "' in '" + pName + "'");
                    res = false;
                }
            }
        }
	    return res;
	}
	
	/* check that an element only has attributes with given names */
	public boolean checkAttributes(Element el,String[] allowed)
	{
	    int i;
	    boolean res = true;
	    NamedNodeMap atts;
	    Attr att;
	    String pName;
	    
        pName = getLocName(el);
	    atts = el.getAttributes();
	    for (i = 0; i < atts.getLength(); i++)
	    {
	        att = (Attr)atts.item(i);
	        if (!inArray(att.getName(),allowed))
	        {
                    message("Unexpected attribute '" + att.getName() + "' in '" + pName + "'");
                    res = false;
	        }
	    }
	    return res;
	}
	
	    
//-------------------------------------------------------------------------------------
//                             Other utilities
//-------------------------------------------------------------------------------------

    	
	/** check if a string is in an array of strings, checking all elements */
	public boolean inArray(String arg, String[] array)
	{
	    int i,len;
	    boolean res = false;
	    len = array.length;
	    if (len > 0) for ( i = 0; i < len; i++)
	        {if (arg.equals(array[i])) res = true;}
	    return res;
	}
	
	/** convert an XPath into a Vector of Strings for nodes in the path separated by '/', 
    outermost node first.
	Always return some Vector - with one element if there are no '/' in the path. */
	Vector pathVector(String XPath)
	{
	    Vector res = new Vector();
	    if (innerNode(XPath).equals(XPath)) {res.addElement(XPath);}
	    else
	    {
	        res = pathVector(removeInnerNode(XPath));
	        res.addElement(innerNode(XPath));
	    }
	    return res;
	}
	
	String outerNode(Vector pv)
	    {return (String)pv.elementAt(0);}
     
    /** find the innermost node of a path represented as n1/n2/n3 */
    String innerNode(String nodePath)
    {
        int i,l,last;
        boolean found;
        String res;
        
        res = "";
        found = false;
        l = nodePath.length();
        // find the position of the last '/' character
        last = 0;
        for (i = 0; i < l; i++)
            if (nodePath.charAt(i) == '/') {last = i; found = true;}
        // substring after that character
        if ((found) && (last < l-1)) {res = nodePath.substring(last+1);}
        else {res = nodePath;}
        return res;
    }
    
    /** remove the innermost element of a path represented as n1/n2/n3 
    converting it to n1/n2.   n3 might be an attribute node such as '@name'. 
    If the path contains no '/', return the whole path. */
    String removeInnerNode(String nodePath)
    {
        int i,l,last;
        boolean found;
        String res;
        
        res = "";
        found = false;
        l = nodePath.length();
        // find the position of the last '/' character
        last = 0;
        for (i = 0; i < l; i++)
            if (nodePath.charAt(i) == '/') {last = i; found = true;}
        // substring before that character
        if ((found) && (last < l-1)) {res = nodePath.substring(0,last);}
        else {res = nodePath;}
        return res;
    }
    
    /* check that the XPath cross is a valid path from node 1 to node 2,
    where node 1 is reached by path1 from the root and node 2 is reached by path2.
    */
    boolean checkCrossPath(String path1, String path2, String cross)
    {
        boolean ancestorFound,validApex;
        int i,apexNode;
        String apex = null;
        String d1,d2;
        Vector p1,p2,c;
        boolean res = false;
        
        p1 = pathVector(path1);
        p2 = pathVector(path2);
        c = pathVector(cross);
        
        // check 'up' path from start node. Short forms not allowed.
        validApex = false;
        if (cross.startsWith("parent::"))
        {
            apex = outerNode(c).substring(8); // strip off string 'parent::'
            if ((p1.size() > 1) // p1 is long enough to define a parent
                && (apex.equals((String)p1.elementAt(p1.size()-2)))) // the parent is as expected from c
                {validApex = true;}
            else {message("Parent error: '" + apex + "' is not a parent of '" 
                + (String)p1.elementAt(p1.size()-1)+ "'");}
        }
        else if (cross.startsWith("ancestor::"))
        {
            ancestorFound = false;
            apex = outerNode(c).substring(10); // strip off string 'ancestor::'
            if (p1.size() > 1) // p1 is long enough to define an ancestor
               for (i = 0; i < p1.size()-2; i++)
                    {if (((String)p1.elementAt(i)).equals(apex)) ancestorFound = true;}
            if (!ancestorFound) {message("Ancestor error: '" + apex + "' is not an ancestor of '" 
                + (String)p1.elementAt(p1.size()-1)+ "'");}
            else {validApex = true;}
       }
       else if (cross.equals("."))
       {
            if (path1.equals(path2)) {return true;}
            else {message("'.' cross path is not allowed"
                + " when node paths from root are not equal.");}
       }
        else
        {
            apex = outerNode(c);
            validApex = true;
        }
        
        // check down path matches end of path2
        if (validApex)
        {
            apexNode = p2.size() - c.size(); // position of apex in p2 path vector
            if (apexNode > -1)
            {
                d2 = (String)p2.elementAt(apexNode);
                if (apex.equals(d2))
                {
                    res = true;
                    for (i = apexNode+1; i < p2.size(); i++)
                    {
                        d1 = (String)c.elementAt(i-apexNode);
                        d2 = (String)p2.elementAt(i);
                        if (!(d1.equals(d2)))
                        {
                            res = false;
                            message("Down path nodes '" + d1 + "' and '"
                                + d2 + "' do not match.");
                        }
                    }
                }
                else {message("Apex node '" + apex 
                    + "' does not match down path node '" + d2 + "'");}
            }
            else {message("Down path length " + c.size() + " is too big.");}
        }
        return res;        
    }
   
    /** 
    * convert a sequence of words into 'java-like' form -
    * remove the gaps between words 
    * and make the initial letter of all words except the first upper case.
    */
    public String gaplessForm(String name)
    {
        int i;
        char[] next = new char[1];
        boolean gap = false;
        String result = "";
        String inc;
        
        for (i = 0; i < name.length(); i++)
        {
            if (name.charAt(i) == ' ')
                {gap = true;}
            else
                {
                    next[0] = name.charAt(i);
                    inc = new String(next);
                    if (gap) {inc = inc.toUpperCase();}
                    gap = false;
                    result = result + inc;
                }
        }
        return result;
    }
    
    // returns true if s1 contains s2 as a substring anywhere
    boolean contains(String s1, String s2)
    {
        int offset;
        boolean res = false;
        if (s1.length() > s2.length() - 1)
            for (offset = 0; offset < s1.length() - s2.length() + 1; offset++)
                if (s1.startsWith(s2,offset)) {res = true;}
        return res;
    }
    
    // make a clone of a vector, for extension without extending the original
    Vector vClone(Vector v)
    {
        int i;
        Vector res = new Vector();
        if (v == null) {message("Attempt to clone a null vector");}
        else for (i=0; i < v.size(); i++) 
            {res.addElement(v.elementAt(i));}
        return res;
    }
    
    /* return a vecotr with all the lements of two input vectors, without
    altering either input vector.*/
    Vector vAdd(Vector v1, Vector v2)
    {
        int i;
        Vector res = new Vector();
        for (i = 0; i < v1.size(); i++)
            {res.addElement(v1.elementAt(i));}
        for (i = 0; i < v2.size(); i++)
            {res.addElement(v2.elementAt(i));}
        return res;
    }

	void message(String s) {System.out.println(s);}
    void tmessage(String s) {if (testing) System.out.println(s);}
    
}