import java.util.*;

public class classDecl
{
    private DAML daml;
    public String className;
    public String superClassName;  //single inheritance class model
    public String comment;
    
    private Vector properties; // propertyDecl objects for this class (not inherited)
    private Vector leftAssociations; /* associationDecl objects where this is the
    left-hand class (class 1) (not inherited) */
    private Vector rightAssociations; /* associationDecl objects where this is the
    right-hand class (class 2) (not inherited) */
    
    private Vector childClasses; // classDecls for immediate subclasses of this class
    
    classDecl(String cName, DAML d)
    {
        className = cName;
        superClassName = null;
        comment = null;
        daml = d;
        childClasses = new Vector();
        properties = new Vector();
        leftAssociations = new Vector();
        rightAssociations = new Vector();
    }
    
    public void addProperty(propertyDecl pd) {properties.addElement(pd);}

    public void addLeftAssociation(associationDecl ad) {leftAssociations.addElement(ad);}
    public void addRightAssociation(associationDecl ad) {rightAssociations.addElement(ad);}

    public void addChildClass(classDecl cd) {childClasses.addElement(cd);}
    
    /** return a Vector of classDecl objects for all subclasses of this class, 
    direct or indirect, including this class itself. */
    public Vector subClasses()
    {
        classDecl cd;
        int i;
        Vector res = new Vector();
        res.addElement(this);
        for (i = 0; i < childClasses.size(); i++)
        {
            cd = (classDecl)childClasses.elementAt(i);
            res = daml.vAdd(res,cd.subClasses());
        }
        return res;
    }
    
    /* return a Vector of propertyDecl objects for all properties, whether declared
     for this class or inherited. 
     Inherited property declarations have the className of the superclass*/
    public Vector allProperties()
    {
        classDecl sc;
        Vector res = properties;
        sc = daml.getClass(superClassName);
        if (sc != null)
            {res = daml.vAdd(res, sc.allProperties());}
        return res;
    }
    
    /* associationDecl objects for all associations in which this is the left
    object (class 1) but not applying inheritance in the right class 2. 
    In these association Decl object, the left class may be a superclass
    of this class. */
    Vector inheritedLeftAssociations()
    {
        classDecl sc;
        Vector res = leftAssociations;
        sc = daml.getClass(superClassName);
        if (sc != null)
            {res = daml.vAdd(res, sc.inheritedLeftAssociations());}
        return res;
    }
    
        
    /* associationDecl objects for all associations in which this is the right
    object (class 2) but not applying inheritance in the left class 1. 
    In these association Decl object, the left class may be a superclass
    of this class. */
    Vector inheritedRightAssociations()
    {
        classDecl sc;
        Vector res = rightAssociations;
        sc = daml.getClass(superClassName);
        if (sc != null)
            {res = daml.vAdd(res, sc.inheritedRightAssociations());}
        return res;
    }
    
    /* all associations in which this is the left hand object (class 1),
    inheriting from superclasses on the left hand side 
    and applying to all subclasses on the right hand side */
    Vector allLeftAssociations()
    {
        associationDecl ad;
        int i;
        Vector la = inheritedLeftAssociations();
        Vector res = new Vector();
        for (i = 0; i < la.size(); i++)
        {
            ad = (associationDecl)la.elementAt(i);
            res = daml.vAdd(res,ad.rightDescendantAssociations());
        }
        return res;
    }
    
    /* all associations in which this is the right hand object (class 2),
    inheriting from superclasses on the right hand side 
    and applying to all subclasses on the left hand side */
    Vector allRightAssociations()
    {
        associationDecl ad;
        int i;
        Vector ra = inheritedRightAssociations();
        Vector res = new Vector();
        for (i = 0; i < ra.size(); i++)
        {
            ad = (associationDecl)ra.elementAt(i);
            res = daml.vAdd(res,ad.leftDescendantAssociations());
        }
        return res;
    }

}