import java.io.*;
import java.util.*;

public class classWriter
{
    private DAML daml;
    private FileOutputStream fo;
    private Vector selectedClasses;
    private byte[] newLine = {13,10};
    
    classWriter(DAML d,Vector sc)
    {
        daml = d;
        selectedClasses = sc;
    }
    
    // true if this class has been selected for code generation
    private boolean selected(String cName)
    {
        boolean res = false;
        int i;
        for (i = 0 ; i < selectedClasses.size(); i++)
            { if (cName.equals((String) selectedClasses.elementAt(i))) res = true;}
        return res;
    }
    
    public void writeAllSource()
    {
        classDecl cd;
        int i;
        try{
            fo = new FileOutputStream("XFactory.java");
        }
        catch (IOException e) {daml.message("File opening exception: " + e.getMessage());}
        writeFactoryHeader();
        for (i = 0; i < daml.classDecls().size(); i++)
        {
            cd = (classDecl)daml.classDecls().elementAt(i);
            if (selected(cd.className)) {writeObjectGetter(cd);}
        }
        writeFactoryFooter();
        try{
            fo.close();
        }
        catch (IOException e) {daml.message("File closing exception: " + e.getMessage());}
        for (i = 0; i < daml.classDecls().size(); i++)
        {
            cd = (classDecl)daml.classDecls().elementAt(i);
            if (selected(cd.className)) {writeClassSource(cd);}
        }
    }
    
    private void writeFactoryHeader()
    {
nl("import org.w3c.dom.*;");
nl("import org.xml.sax.*;");
nl("import java.util.*;");
nl("");
nl("/* represents a set of objects in the classes of a DAML/UML semantic model.");
nl("This is the set of objects represented in some XML document.");
nl("This class is a bunch of factory methods to return every object of any given class.");
nl("*/");
nl("public class XFactory");
nl("{");
nl("    Node root; // root of XML document");
nl("    MDL mdl; // internal representation of the MDL for the XML document");
nl(" ");   
nl("    public XFactory(Node r, MDL m)");
nl("    {");
nl("        root = r;");
nl("        mdl = m;");
nl("    }");
nl("");
    }
    
    private void writeObjectGetter(classDecl cd)
    {
        String cName = cd.className;
    nl("");
    nl("/* return a vector of all '" + cName + "' objects represented in the document;");
    nl("or null if the language does not represent objects of this class. */");
    nl("public Vector every_" + cName + "()");
    nl("{");
    nl("    int i;");
    nl("    Vector res = null;");
    nnl("    Vector nl = mdl.getAllObjectNodes(");
    quoted(cName);
    nl(", root);");
    nl("    if (nl != null) {");
    nl("        res = new Vector();");
    nl("        for (i = 0; i < nl.size(); i++)");
    nl("            {res.addElement(new " + cName + "((Node)nl.elementAt(i),mdl));}");
    nl("    }");
    nl("    return res;");
    nl("}");
    }
    
    private void writeFactoryFooter()
    {
        nl("}");
    }
    
    // write out a String followed by a new line
    private void nl(String s)
    {
        try{
        fo.write(s.getBytes());
        fo.write(newLine);
        }
        catch (IOException e) 
            {daml.message("Exception writing line '" + s + "': " + e.getMessage());}
    }
    
    // write out a String followed by no new line
    private void nnl(String s)
    {
        try{
        fo.write(s.getBytes());
        }
        catch (IOException e) 
            {daml.message("Exception writing line '" + s + "': " + e.getMessage());}
    }
    
    // write out a string enclosed by double quote '"' characters
    private void quoted(String s)
    {
        byte[] quote = {'"'};
        try{
            fo.write(quote);
            fo.write(s.getBytes());
            fo.write(quote);
        }
        catch (IOException e) 
            {daml.message("Exception writing quoted string '" + s + "': " + e.getMessage());}
    }
    
    private void writeClassSource(classDecl cd)
    {
        propertyDecl pd;
        associationDecl ad;
        String cName = cd.className;
        String sName = cd.superClassName;
        String fName = cName + ".java";
        int i;
        try{
            fo = new FileOutputStream(fName);
        }
        catch (IOException e) 
         {daml.message("Exception opening " + fName + " : " + e.getMessage());}
        writeClassHeader(cName,sName);
        
        // properties of the class
        for (i = 0; i < cd.allProperties().size(); i++)
        {
            pd  = (propertyDecl)cd.allProperties().elementAt(i);
            writeClassProperty(cName,pd);
        }
        
        // associations in which the class is class 1 (left-hand class)
        for (i = 0; i < cd.allLeftAssociations().size(); i++)
        {
            ad  = (associationDecl)cd.allLeftAssociations().elementAt(i);
            if (selected(ad.class2Name)) 
            {
                if (ad.class2Unique)
                    {writeSingleLeftAssociation(cName,ad);}
                else
                    {writeMultipleLeftAssociation(cName,ad);}
            }
        }
        
        // associations in which the class is class 2 (RIGHT-hand class)
        for (i = 0; i < cd.allRightAssociations().size(); i++)
        {
            ad  = (associationDecl)cd.allRightAssociations().elementAt(i);
            if (selected(ad.class1Name)) 
            {
                 if (ad.class1Unique)
                    {writeSingleRightAssociation(cName,ad);}
                else
                    {writeMultipleRightAssociation(cName,ad);}
            }
        }
        
        writeClassFooter();
        try{
            fo.close();
        }
        catch (IOException e) {daml.message("File closing exception: " + e.getMessage());}
    }
    
    private void writeClassHeader(String cName,String sName)
    {
    nl("import org.w3c.dom.*;");
    nl("import java.util.*;");
    nl("");
    if (selected(sName))
        {nl("public class " + cName + " extends " + sName);}
    else
        {nl("public class " + cName);}
    nl("{");
    if (!selected(sName)) // do not need to declare these if they are inherited from superclass
    {
        nl("    protected Node objectNode;");
        nl("    protected MDL mdl;");
    }
    nl("");   
    nl("    public " + cName + "(Node n, MDL m)");
    if (selected(sName)) // if there is a superclass, use its constructor
        {nl("        {super(n,m);}");}
    else
        {nl("        {objectNode = n; mdl = m;}");}    
    }
    
    private void writeClassProperty(String cName,propertyDecl pd)
    {
    String pName = pd.propertyName;
    nl("");   
    nl("    // String value of '" + pName + "' property");
    nl("    public String " + pName + "()");
    nnl("        {return mdl.getPropertyValue(");
    quoted(cName);
    nnl(",");
    quoted(pName);
    nl(",objectNode,false);}");
    }

    private void writeSingleLeftAssociation(String cName, associationDecl ad)
    {
        String rClass = ad.class2Name;
        String aName = ad.associationName;
        nl("");    
        nl("// Single '" + rClass + "' object related by ["
            + cName + "]" + aName + "[" + rClass + "]");
        nl("public " + rClass + " " + aName + "_" + rClass + "()");
        nl("{");
        nl("    " + rClass + " res = null;");
        nnl("    Vector nl = mdl.getAssociatedObjectNodes(");
        quoted(cName);nnl(",");
        quoted(aName);nnl(",");
        quoted(rClass);nnl(",");
        nl("objectNode,1);");
        nl("    if ((nl != null) && (nl.size() > 0))" 
            + " {res = new " + rClass + "((Node)nl.elementAt(0),mdl);}");
        nl("    return res;");
        nl("}");
    }

    private void writeSingleRightAssociation(String cName, associationDecl ad)
    {
        String lClass = ad.class1Name;
        String aName = ad.associationName;
        nl("");    
        nl("// Single '" + lClass + "' object related by ["
            + lClass + "]" + aName + "[" + cName + "]");
        nl("public " + lClass + " " + lClass + "_" + aName + "()");
        nl("{");
        nl("    " + lClass + " res = null;");
        nnl("    Vector nl = mdl.getAssociatedObjectNodes(");
        quoted(lClass);nnl(",");
        quoted(aName);nnl(",");
        quoted(cName);nnl(",");
        nl("objectNode,2);");
        nl("    if ((nl != null) && (nl.size() > 0))" 
            + " {res = new " + lClass + "((Node)nl.elementAt(0),mdl);}");
        nl("    return res;");
        nl("}");
    }
    
    
    private void writeMultipleLeftAssociation(String cName, associationDecl ad)
    {
        String rClass = ad.class2Name;
        String aName = ad.associationName;
        nl("");    
        nl("// Vector of '" + rClass + "' objects related by ["
            + cName + "]" + aName + "[" + rClass + "]");
        nl("public Vector " + aName + "_" + rClass + "()");
        nl("{");
        nl("    Vector res = null;");
        nnl("    Vector nl = mdl.getAssociatedObjectNodes(");
        quoted(cName);nnl(",");
        quoted(aName);nnl(",");
        quoted(rClass);nnl(",");
        nl("objectNode,1);");
        nl("    if (nl != null) {res = new Vector();");
        nl("       for (int i = 0; i < nl.size(); i++)");
        nl("        {res.addElement(new " + rClass + "((Node)nl.elementAt(i),mdl));}}");
        nl("    return res;");
        nl("}");
    }
    
    private void writeMultipleRightAssociation(String cName, associationDecl ad)
    {
        String lClass = ad.class1Name;
        String aName = ad.associationName;
        nl("");    
        nl("// Vector of '" + lClass + "' objects related by ["
            + lClass + "]" + aName + "[" + cName + "]");
        nl("public Vector " + lClass + "_" + aName + "()");
        nl("{");
        nl("    Vector res = null;");
        nnl("    Vector nl = mdl.getAssociatedObjectNodes(");
        quoted(lClass);nnl(",");
        quoted(aName);nnl(",");
        quoted(cName);nnl(",");
        nl("objectNode,2);");
        nl("    if (nl != null) {res = new Vector();");
        nl("       for (int i = 0; i < nl.size(); i++)");
        nl("        {res.addElement(new " + lClass + "((Node)nl.elementAt(i),mdl));}}");
        nl("    return res;");
        nl("}");
    }
    
    private void writeClassFooter()
    {
        nl("}");
    }
}