import org.w3c.dom.*;
import org.xml.sax.*;
import java.util.*;

/**
 *  Demonstration program for meaning-level Java API to XML
 *  Takes 2 arguments:
 *     (1) an XML filename
 *     (2) filename for the MDL definition of that XML
 *  Runs the application code in method application() against the XML,
 *  using the MDL to get the required inforoamtion from the XML.
 */
public class poApp
{
  protected String XMLFileName = null;
  protected String MDLFileName = null;
  
  protected MDL mdl = null;
  protected XMLIO xio;
  protected XFactory xf;  

  /** Process input args and open the XML file  */
  public void doMain(String[] args)
  {

    XMLFileName = args[0];
    MDLFileName = args[1];

    if ((XMLFileName != null) && (XMLFileName.length() > 0)
        && (MDLFileName != null) && (MDLFileName.length() > 0))
    {
      
      //   open MDL file and process it
      mdl = new MDL(MDLFileName,false); // no detailed trace info
      mdl.captureMDL();
      message("MDL read in;");
      
      if (mdl.valid())
      {
        // read XML input file
        xio = new XMLIO(false);
        Element root = xio.readXMLFile(XMLFileName);
        message("XML file read in;");
        
        // create factory for objects represented in the XML
        xf = new XFactory(root,mdl);
        
        // run the application code
        application(xf);
      }
      else {message("Application not run because of MDL errors.");}
    }
    else
    {
      message("Bad input args: " + XMLFileName + ", " + MDLFileName);
    }
  }
  
  /** this is the actual application code, written in terms of application domain
  classes. */
  void application(XFactory xf)
  {
    /* java code to reproduce the query:
    
    display recipientUnit.name, shippingAddress.city 
    where recipientUnit hasPostAddress shippingAddress

     this query has a join and produces a non-trivial result against 
     many of the 13 different purchase order messages.
     */
     
     int i, j;
     recipientUnit ru;
     shippingAddress sa;
     
     Vector recipients = xf.every_recipientUnit();
     for (i = 0; i < recipients.size(); i++)
     {
        ru = (recipientUnit)recipients.elementAt(i);
        // note the DAML model allows one recipient unit to have several addresses
        Vector addresses = ru.hasPostAddress_shippingAddress();
        for (j = 0; j < addresses.size(); j++)
        {
            sa = (shippingAddress)addresses.elementAt(j);
            message("Recipient name: " + ru.name() + "  Address city: " + sa.city());
        }
     }
  }
    
  /** Main method to run from the command line.    */
  public static void main (String[] args)
  {
    if (args.length != 2)
    {
      System.out.println("java poApp MDLFileName XMLFileName"
                         + " runs the application poApp"
                         + " against XMLFileName"
                         + " whose MDL is in MDLFileName.");
      return;
    }
    poApp app = new poApp();
    app.doMain(args);
  }

void message(String s) {System.out.println(s);}


}